SystemOrganization addCategory: #TusLibros!

TestCase subclass: #TusLibrosTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TusLibrosTest methodsFor: 'accessing' stamp: 'KevinKujawski 5/29/2013 21:05'!
catalogue
 	| catalogue |
	catalogue := Dictionary new.
	catalogue add: (Association key: 1 value: 1).
	catalogue add: (Association key: 2 value: 2).
	^ catalogue.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 20:16'!
clientbook
	^Clientbook create.! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 20:25'!
clientbookNotEmpty
	^(Clientbook create)addClient: 1 withPassword: 2.! !

!TusLibrosTest methodsFor: 'clock provider' stamp: 'KevinKujawski 6/9/2013 23:21'!
clock
	^ (ClockStub new)return:1.
! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/3/2013 20:11'!
creditCard
	^ Card newNumber: 1231231212 expirationMonth: 08 expirationYear: 2013 owner: 'Manolo'.! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 5/29/2013 20:16'!
test01NewCartShouldBeEmpty
	| newCart aCatalogue|
	aCatalogue := Array new.
	newCart := Cart withCatalogue: aCatalogue .
	self assert: newCart listProducts = Bag new.! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 5/29/2013 20:49'!
test02CartShouldListProductAfterAdded
	| newCart  bag|
	newCart := Cart withCatalogue: self catalogue .	
		
	newCart add: 1.
	bag := newCart listProducts.
	self assert: (bag includes: 1 ).
	self assert: (bag size = 1 ).! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 5/29/2013 20:28'!
test02bQuantityShouldIncreaseInOne
	| newCart firstQuantity |
	newCart := Cart withCatalogue: self catalogue.
	firstQuantity := newCart quantityOf: 1.
	newCart add: 1.
	self assert: ((firstQuantity + 1) = (newCart quantityOf: 1)).! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/3/2013 16:28'!
test03CannotAddProductNotInCatalogue
	| newCart |
	newCart := Cart withCatalogue: self catalogue.
	self should: [ newCart add:15 ] raise: Error withExceptionDo: [ :anError | self assert: (anError messageText   = Cart invalidElementError ). self assert: ((newCart listProducts) isEmpty ) ].! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/3/2013 16:29'!
test04IfModifyListProductNotModifyCartProduct
	|bagToModify newCart |
	newCart := Cart withCatalogue: self catalogue .	
		
	newCart add: 1.
	bagToModify := newCart listProducts.
	bagToModify removeAll.
	self assert: ( newCart listProducts includes: 1 ).
	self assert: ( newCart listProducts size = 1 ).! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 22:25'!
test05EmptyCartCheckoutShouldMakeAnException
	| newCart newCashier aCreditCard aMerchantProcessor |
	newCart := Cart withCatalogue: self catalogue.
	newCashier := Cashier newWithDateToday: (Date newDay: 01 month: 09 year: 2012).
	aCreditCard := self creditCard.
	aMerchantProcessor := MerchantProcessorMock create.
	self
		should: [ newCashier checkout: newCart card: aCreditCard withMerchantProcessor: aMerchantProcessor ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText = Cashier emptyCartError.
			self assert: aMerchantProcessor connections = 0 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 22:25'!
test06ExpiredCardCheckoutShouldMakeAnException
	| newCart newCashier aCreditCard aMerchantProcessor |
	newCart := Cart withCatalogue: self catalogue.
	newCart add: 1.
	newCashier := Cashier newWithDateToday: (Date newDay: 01 month: 09 year: 2014).
	aCreditCard := self creditCard.
	aMerchantProcessor := MerchantProcessorMock create.
	self
		should: [ newCashier checkout: newCart card: aCreditCard withMerchantProcessor: aMerchantProcessor ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText = Card expiredCardError.
			self assert: aMerchantProcessor connections = 0 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 22:25'!
test07CreditCardWithNoCreditShouldReturnAnError
	| newCart newCashier aCreditCard aMerchantProcessor |
	newCart := Cart withCatalogue: self catalogue.
	newCart add: 1.
	newCashier := Cashier newWithDateToday: (Date newDay: 01 month: 09 year: 2010).
	aCreditCard := self creditCard.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [ Error signal: Cashier noCreditError ].
	self
		should: [ newCashier checkout: newCart card: aCreditCard withMerchantProcessor: aMerchantProcessor ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText = Cashier noCreditError.
			self assert: aMerchantProcessor connections = 1 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 22:25'!
test08CreditCardWithCreditShouldIncrementTheSalesbook
	| newCart newCashier aCreditCard aMerchantProcessor |
	newCart := Cart withCatalogue: self catalogue.
	newCart add: 1.
	newCashier := Cashier newWithDateToday: (Date newDay: 01 month: 09 year: 2010).
	aCreditCard := self creditCard.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [  ].
	newCashier checkout: newCart card: aCreditCard withMerchantProcessor: aMerchantProcessor.
	self assert: newCashier salesbook size = 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 22:25'!
test09StolenCreditCardShouldReturnAnError
	| newCart newCashier aCreditCard aMerchantProcessor |
	newCart := Cart withCatalogue: self catalogue.
	newCart add: 1.
	newCashier := Cashier newWithDateToday: (Date newDay: 01 month: 09 year: 2010).
	aCreditCard := self creditCard.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [ Error signal: Cashier stolenCardError ].
	self
		should: [ newCashier checkout: newCart card: aCreditCard withMerchantProcessor: aMerchantProcessor ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText = Cashier stolenCardError.
			self assert: aMerchantProcessor connections = 1 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/5/2013 21:24'!
test10InvalidUserShouldReturnAnException
	| newInterface |
	newInterface := Interface withClientbook: self clientbook andCatalogue: self catalogue andClock: nil.
	self
		should: [ newInterface createFor: 'manolo' withPass: 'poo' ]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError messageText = Clientbook invalidClientError ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/5/2013 21:50'!
test11validUserShouldReturnACartId
	| newInterface |
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.
	self assert: (newInterface listCart: (newInterface createFor: 1 withPass: 2)) isEmpty! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 19:02'!
test12invalidCartIdInListCartShouldReturnAnError
	| newInterface |
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: nil.
	self
		should: [ newInterface listCart: 16 ]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError messageText = Interface invalidCartIdError ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/5/2013 21:50'!
test13differentsCartShouldHaveDifferentsId
	| newInterface cartId1 cartId2 |
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.
	cartId1 := newInterface createFor: 1 withPass: 2.
	cartId2 := newInterface createFor: 1 withPass: 2.
	self assert: cartId1 ~= cartId2! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/5/2013 21:50'!
test14inExpiredTimeCartActionShouldReturnError
	| newInterface cartId1 clock|
	clock := self clock.
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: clock.

	cartId1 := newInterface createFor: 1 withPass: 2.
	clock return: 90.
	self
		should: [ newInterface listCart: cartId1 ]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError messageText = Interface expiredTimeError  ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 19:05'!
test15invalidCartIdInAddToCarttShouldReturnAnError
	| newInterface |
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: nil.
	self
		should: [ newInterface addToCart: 16 book: nil  withQuantity:  nil]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError messageText = Interface invalidCartIdError ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 19:27'!
test16invalidISBNInAddToCarttShouldReturnAnError
	| newInterface cartId|
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.
	cartId := newInterface createFor: 1 withPass: 2.
	self
		should: [ newInterface addToCart: cartId book: 3  withQuantity:  1]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError messageText = Cart invalidElementError ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 20:12'!
test17invalidQuantityInAddToCarttShouldReturnAnError
	| newInterface cartId|
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.
	cartId := newInterface createFor: 1 withPass: 2.
	self
		should: [ newInterface addToCart: cartId book: 1  withQuantity:  0]
		raise: Error
		withExceptionDo: [ :anError | self assert: anError messageText = Cart invalidQuantityElementsError  ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 20:48'!
test18AddToCartWithValidDataShouldAddTheProduct
	| newInterface cartId|
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.
	cartId := newInterface createFor: 1 withPass: 2.
	newInterface addToCart: cartId book: 1  withQuantity:  2.
	self assert: ( (newInterface listCart:cartId) includes: 1 ).
	self assert: (  (newInterface listCart:cartId) size = 2 ).! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 23:57'!
test19CheckOutInterfaceWithCreditCardWithNoCreditShouldReturnAnError
	| newCartId newInterface aMerchantProcessor anExpirationDate |
	
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.	
	newCartId :=newInterface createFor: 1 withPass: 2.
	newInterface addToCart: newCartId book:  1 withQuantity: 1.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [ Error signal: Cashier noCreditError ].  
	anExpirationDate:= Date newDay: 1 month: self creditCard monthExpiration  year: self creditCard yearExpiration .
	self
		should: [ newInterface checkOut: newCartId withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2010). ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText = Cashier noCreditError.
			self assert: aMerchantProcessor connections = 1 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 23:58'!
test20CheckOutInterfaceWithExpiredCardCheckoutShouldMakeAnException
	| newCartId newInterface aMerchantProcessor anExpirationDate |
	
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.	
	newCartId :=newInterface createFor: 1 withPass: 2.
	newInterface addToCart: newCartId book:  1 withQuantity: 1.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [ Error signal: Cashier noCreditError ].  
	anExpirationDate:= Date newDay: 1 month: self creditCard monthExpiration  year: self creditCard yearExpiration .
	self
		should: [ newInterface checkOut: newCartId withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2014). ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText =Card expiredCardError.
			self assert: aMerchantProcessor connections = 0 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 23:58'!
test21CheckOutInterfaceWithStolenCreditCardShouldReturnAnError
	| newCartId newInterface aMerchantProcessor anExpirationDate |
	
	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.	
	newCartId :=newInterface createFor: 1 withPass: 2.
	newInterface addToCart: newCartId book:  1 withQuantity: 1.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [  Error signal: Cashier stolenCardError ].  
	anExpirationDate:= Date newDay: 1 month: self creditCard monthExpiration  year: self creditCard yearExpiration .
	self
		should: [ newInterface checkOut: newCartId withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2014). ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText =Card expiredCardError.
			self assert: aMerchantProcessor connections = 0 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/9/2013 23:58'!
test22CheckOutInterfaceWithEmptyCartCheckoutShouldMakeAnException
	| newCartId newInterface aMerchantProcessor anExpirationDate |

	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.	
	newCartId :=newInterface createFor: 1 withPass: 2.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [  Error signal: Cashier stolenCardError ].  
	anExpirationDate:= Date newDay: 1 month: self creditCard monthExpiration  year: self creditCard yearExpiration .
	self
		should: [ newInterface checkOut: newCartId withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2014). ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText =Cashier emptyCartError.
			self assert: aMerchantProcessor connections = 0 ]! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/10/2013 14:18'!
test23SuccessfullCheckOutInterfaceShouldIncrementTheClientPurchases
	| newCartId newInterface aMerchantProcessor anExpirationDate finalAmount|

	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.	
	newCartId :=newInterface createFor: 1 withPass: 2.
	newInterface addToCart: newCartId book: 1  withQuantity:  2.
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [].  
	anExpirationDate:= Date newDay: 1 month: self creditCard monthExpiration  year: self creditCard yearExpiration .
	finalAmount := newInterface checkOut: newCartId withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2000).
	self assert:((newInterface listPurchasesFor: 1 pass: 2) size = 1).
	self assert:(finalAmount = 2).
	
	newCartId :=newInterface createFor: 1 withPass: 2.
	newInterface addToCart: newCartId book: 1  withQuantity:  10.
	newInterface addToCart: newCartId book: 2  withQuantity:  10.
	finalAmount := newInterface checkOut: newCartId withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2000).
	self assert:((newInterface listPurchasesFor: 1 pass: 2) size = 2).
	self assert:(finalAmount = 30).

		! !

!TusLibrosTest methodsFor: 'tests' stamp: 'KevinKujawski 6/10/2013 01:51'!
test24CheckOutInterfaceWithInvalidCartIdShouldMakeAnException
	| newInterface aMerchantProcessor anExpirationDate |

	newInterface := Interface withClientbook: self clientbookNotEmpty andCatalogue: self catalogue andClock: self clock.	
	aMerchantProcessor := MerchantProcessorMock create.
	aMerchantProcessor forMethod: 'debit' return: [].  
	anExpirationDate:= Date newDay: 1 month: self creditCard monthExpiration  year: self creditCard yearExpiration .
	self
		should: [ newInterface checkOut: -1 withCardNumber: self creditCard number  ExpirationDate: anExpirationDate OwnerName:  self creditCard owner MerchantProcessor: aMerchantProcessor andTodayDate:  (Date newDay: 01 month: 09 year: 2000). ]
		raise: Error
		withExceptionDo: [ :anError | 
			self assert: anError messageText = Interface invalidCartIdError .
			self assert: aMerchantProcessor connections = 0 ]! !

Object subclass: #Card
	instanceVariableNames: 'number monthExpiration yearExpiration owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Card class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 21:01'!
expiredCardError
	^'Tarjeta vencida'.! !

!Card class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/3/2013 20:08'!
newNumber: aCardNumber expirationMonth: aMonth expirationYear: aYear owner: aName 
	^Card new number: aCardNumber expirationMonth: aMonth expirationYear: aYear owner: aName ! !

!Card methodsFor: 'testing' stamp: 'KevinKujawski 6/5/2013 21:01'!
isExpiredOn: aDate
	^(yearExpiration < aDate year) |  (yearExpiration = aDate year and: [monthExpiration < aDate monthIndex]) ifTrue: [ Error signal: Card expiredCardError ]! !

!Card methodsFor: 'accessing' stamp: 'KevinKujawski 6/3/2013 19:08'!
monthExpiration
	^monthExpiration! !

!Card methodsFor: 'accessing' stamp: 'KevinKujawski 6/9/2013 22:51'!
number
	^number.
! !

!Card methodsFor: 'initialize-release' stamp: 'KevinKujawski 6/3/2013 20:09'!
number: aCardNumber expirationMonth: aMonth expirationYear: aYear owner: aName 
	number:=aCardNumber.
	monthExpiration := aMonth.
	yearExpiration := aYear.
	owner := aName.
	! !

!Card methodsFor: 'accessing' stamp: 'KevinKujawski 6/9/2013 22:51'!
owner
	^owner.
! !

!Card methodsFor: 'accessing' stamp: 'KevinKujawski 6/3/2013 19:08'!
yearExpiration
	^yearExpiration
! !

Object subclass: #Cart
	instanceVariableNames: 'bag catalogue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 5/27/2013 21:45'!
create
	^Cart new initialize.! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 5/29/2013 20:42'!
invalidElementError
	^'elemento invalido'.! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/9/2013 20:11'!
invalidQuantityElementsError
	^'Cantidad de elementos a agregar invlida'.! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 5/29/2013 20:22'!
withCatalogue: aCatalogue
	|cart|
      cart  := Cart new.
	cart setCatalogue: aCatalogue.
	^ cart.! !

!Cart methodsFor: 'adding' stamp: 'KevinKujawski 6/9/2013 20:09'!
add: aProduct
	self add: aProduct withQuantity:1.! !

!Cart methodsFor: 'adding' stamp: 'KevinKujawski 6/9/2013 20:10'!
add: aProduct withQuantity:aQuantity
	(catalogue includes: aProduct ) ifFalse: [ Error signal: Cart invalidElementError ].
	(aQuantity <= 0)ifTrue:[Error signal: Cart invalidQuantityElementsError].
	bag add: aProduct withOccurrences: aQuantity.! !

!Cart methodsFor: 'accessing' stamp: 'KevinKujawski 6/3/2013 16:23'!
catalogue
	^catalogue.! !

!Cart methodsFor: 'initialize-release' stamp: 'KevinKujawski 5/27/2013 21:45'!
initialize
	bag := Bag new.! !

!Cart methodsFor: 'testing' stamp: 'KevinKujawski 6/3/2013 18:58'!
isEmpty
	^bag size = 0.! !

!Cart methodsFor: 'accessing' stamp: 'KevinKujawski 5/29/2013 19:24'!
listProducts
	^ Bag newFrom:  bag.! !

!Cart methodsFor: 'accessing' stamp: 'KevinKujawski 5/29/2013 19:24'!
quantityOf: aProduct
	^bag occurrencesOf: aProduct.! !

!Cart methodsFor: 'initialize-release' stamp: 'KevinKujawski 5/29/2013 20:23'!
setCatalogue: aCatalogue
	catalogue := aCatalogue.! !

!Cart methodsFor: 'accessing' stamp: 'KevinKujawski 6/10/2013 14:18'!
totalAmount
	|finalSum|
	finalSum := 0.
	bag do: [ :each | finalSum := finalSum + (catalogue at: each) ].
	^finalSum.! !

Object subclass: #Cashier
	instanceVariableNames: 'salesbook today'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/3/2013 20:15'!
emptyCartError
	^'Carrito vacio'.! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/9/2013 22:19'!
newWithDateToday: aDate
	^ Cashier new newWithDateToday: aDate! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/3/2013 20:50'!
noCreditError
	^'Tarjeta sin credito'.! !

!Cashier class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 19:36'!
stolenCardError
	^'Tarjeta robada'.! !

!Cashier methodsFor: 'validation' stamp: 'KevinKujawski 6/5/2013 21:02'!
checkCardExpiration: aCard
	^aCard isExpiredOn: today.! !

!Cashier methodsFor: 'validation' stamp: 'KevinKujawski 6/3/2013 20:25'!
checkEmptyCart: aCart
	 ( aCart isEmpty ) ifTrue: [ Error signal: (Cashier emptyCartError) ]! !

!Cashier methodsFor: 'validation' stamp: 'KevinKujawski 6/10/2013 14:13'!
checkout: aCart card: aCard withMerchantProcessor: aMerchant
	|finalSum result newSale|
	
	self checkEmptyCart: aCart.
	self checkCardExpiration: aCard.
	finalSum:= aCart totalAmount.
	result := aMerchant debit: finalSum from: aCard.
	newSale := Sale cart: aCart amount: finalSum card: aCard.
	salesbook add: newSale.
	^finalSum.! !

!Cashier methodsFor: 'instance-creation' stamp: 'KevinKujawski 6/9/2013 22:19'!
newWithDateToday: aDate 
	salesbook := Set new.
	today := aDate.! !

!Cashier methodsFor: 'accessing' stamp: 'KevinKujawski 6/5/2013 13:39'!
salesbook
	^ salesbook! !

Object subclass: #Clientbook
	instanceVariableNames: 'clients'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Clientbook class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 20:16'!
create
	^Clientbook new initialize .! !

!Clientbook class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 20:14'!
invalidClientError
	^'Cliente invalido'.! !

!Clientbook methodsFor: 'adding' stamp: 'KevinKujawski 6/5/2013 20:24'!
addClient:aClient withPassword: aPassword
	clients add:(Association key: aClient value: aPassword)
	! !

!Clientbook methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 20:42'!
existClient: aClient withPassword: aPassword	
		(clients includesKey: aClient) ifFalse: [ Error signal: Clientbook invalidClientError. ].
		((clients at:aClient) = aPassword) ifFalse: [ Error signal: Clientbook invalidClientError. ]! !

!Clientbook methodsFor: 'initialize-release' stamp: 'KevinKujawski 6/5/2013 20:16'!
initialize
	clients:=Dictionary new.! !

Object subclass: #ClockStub
	instanceVariableNames: 'calls valueToReturn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!ClockStub methodsFor: 'handling' stamp: 'KevinKujawski 6/5/2013 21:35'!
return: aValue
	valueToReturn := aValue.
	! !

!ClockStub methodsFor: 'accessing' stamp: 'KevinKujawski 6/5/2013 21:35'!
time
	^valueToReturn.
	! !

Object subclass: #Interface
	instanceVariableNames: 'sessions clientBook catalogue clock salesBook'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Interface class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 21:37'!
expiredTimeError
	^'Paso el tiempo'! !

!Interface class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 21:12'!
invalidCartIdError
	^'Id de carrito invalido'! !

!Interface class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 21:46'!
withClientbook: aClientbook andCatalogue: aCatalogue andClock: aClock
	^ Interface new withClientbook: aClientbook andCatalogue: aCatalogue andClock: aClock! !

!Interface methodsFor: 'adding' stamp: 'KevinKujawski 6/10/2013 00:12'!
addToCart: aCartId book: aISBNBook withQuantity: aQuantity
 	self validateCartId: aCartId.
	self validateTimeAlive: aCartId.
	((sessions at: aCartId) cart) add: aISBNBook withQuantity: aQuantity.




! !

!Interface methodsFor: 'interface' stamp: 'KevinKujawski 6/10/2013 14:18'!
checkOut: aCartId withCardNumber: aCardNumber ExpirationDate: aExpirationDate OwnerName: aOwnerName MerchantProcessor: aMerchantProcessor andTodayDate: aTodayDate
	|aCard aCashier results cart|
	self validateCartId: aCartId.
	self validateTimeAlive: aCartId.
	aCard := Card newNumber: aCardNumber expirationMonth: (aExpirationDate month) expirationYear: (aExpirationDate year) owner: aOwnerName.
	aCashier := Cashier newWithDateToday: aTodayDate.
	cart:=((sessions at: aCartId) cart) .
	results :=aCashier checkout: cart card: aCard withMerchantProcessor: aMerchantProcessor.
	salesBook add: aCartId.
	^results.
	! !

!Interface methodsFor: 'interface' stamp: 'KevinKujawski 6/9/2013 23:29'!
createFor: aClient withPass: aPassword 

   clientBook existClient: aClient withPassword: aPassword.
   sessions add:( Session cart: (Cart withCatalogue: catalogue) user: aClient time: (clock time)).
   ^sessions size.




! !

!Interface methodsFor: 'interface' stamp: 'KevinKujawski 6/10/2013 00:01'!
listCart: aCartId
	self validateCartId: aCartId.
	(clock time - (sessions at: aCartId) time  > 30) ifTrue:[^ Error signal: (Interface expiredTimeError) ].
	^(sessions at: aCartId) cart listProducts .! !

!Interface methodsFor: 'interface' stamp: 'KevinKujawski 6/10/2013 01:11'!
listPurchasesFor: aClient pass: aPassword
	|clientCarts|
	clientBook existClient: aClient withPassword: aPassword.
	clientCarts := Set new.
	salesBook do: [ :each | ((sessions at: each) user = aClient) ifTrue:[clientCarts add: ((sessions at: each)cart)]].
	^clientCarts.
! !

!Interface methodsFor: 'validation' stamp: 'KevinKujawski 6/10/2013 01:56'!
validateCartId: aCartId
	(sessions size < aCartId | (aCartId < 0)) ifTrue:[^ Error signal: (Interface invalidCartIdError) ].
	! !

!Interface methodsFor: 'validation' stamp: 'KevinKujawski 6/10/2013 00:13'!
validateTimeAlive: aCartId
	(clock time - (sessions at: aCartId) time  > 30) ifTrue:[^ Error signal: (Interface expiredTimeError) ].
! !

!Interface methodsFor: 'initialize-release' stamp: 'KevinKujawski 6/10/2013 00:21'!
withClientbook: aClientbook andCatalogue: aCatalogue andClock: aClock.
	clientBook := aClientbook.
	sessions := OrderedCollection new.
	catalogue := aCatalogue.
	clock :=  aClock.
	salesBook := Set new.! !

Object subclass: #MerchantProcessorMock
	instanceVariableNames: 'valueToReturn connections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!MerchantProcessorMock class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/3/2013 20:52'!
create
	^MerchantProcessorMock new initialize.! !

!MerchantProcessorMock methodsFor: 'accessing' stamp: 'KevinKujawski 6/3/2013 21:05'!
connections
	^connections.! !

!MerchantProcessorMock methodsFor: 'action' stamp: 'KevinKujawski 6/3/2013 21:09'!
debit: anAmount from: aCard
	connections := connections + 1.
	^(valueToReturn at: 'debit') value. ! !

!MerchantProcessorMock methodsFor: 'mock' stamp: 'KevinKujawski 6/3/2013 20:54'!
forMethod: aMethod return: aValue
	valueToReturn add: (Association key: aMethod value: aValue )! !

!MerchantProcessorMock methodsFor: 'initialize-release' stamp: 'KevinKujawski 6/3/2013 21:05'!
initialize
	valueToReturn := Dictionary new.
	connections := 0.! !

Object subclass: #Sale
	instanceVariableNames: 'cart amount card'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sale class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 13:54'!
cart:aCart amount: anAmount card: aCard 
	^Sale new cart:aCart amount: anAmount card: aCard.! !

!Sale methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/5/2013 13:54'!
cart:aCart amount: anAmount card: aCard 
	cart:=aCart.
	amount:=anAmount.
	card:= aCard.! !

Object subclass: #Session
	instanceVariableNames: 'cart user time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Session class methodsFor: 'as yet unclassified' stamp: 'KevinKujawski 6/9/2013 23:08'!
cart: aCart user: anUser time: aTime
	^Session new cart: aCart user: anUser time:aTime! !

!Session methodsFor: 'accessing' stamp: 'KevinKujawski 6/9/2013 23:15'!
cart
	^cart.! !

!Session methodsFor: 'initialize-release' stamp: 'KevinKujawski 6/9/2013 23:06'!
cart: aCart user: anUser time: aTime
	cart:=aCart.
	user :=anUser.
	time:=aTime.! !

!Session methodsFor: 'accessing' stamp: 'KevinKujawski 6/9/2013 23:11'!
time
	^time.! !

!Session methodsFor: 'accessing' stamp: 'KevinKujawski 6/9/2013 23:16'!
user
	^user.! !
